"use strict";
/* eslint-disable no-param-reassign, no-underscore-dangle */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const debug_1 = tslib_1.__importDefault(require("debug"));
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const docGen = tslib_1.__importStar(require("react-docgen-typescript"));
const micromatch_1 = require("micromatch");
const dependency_1 = tslib_1.__importDefault(require("./dependency"));
const generateDocgenCodeBlock_1 = require("./generateDocgenCodeBlock");
const debugExclude = debug_1.default("docgen:exclude");
/** Get the contents of the tsconfig in the system */
function getTSConfigFile(tsconfigPath) {
    try {
        const basePath = path_1.default.dirname(tsconfigPath);
        const configFile = typescript_1.default.readConfigFile(tsconfigPath, typescript_1.default.sys.readFile);
        return typescript_1.default.parseJsonConfigFileContent(configFile.config, typescript_1.default.sys, basePath, {}, tsconfigPath);
    }
    catch (error) {
        return {};
    }
}
/** Create a glob matching function. */
const matchGlob = (globs) => {
    const matchers = (globs || []).map((g) => micromatch_1.matcher(g));
    return (filename) => Boolean(filename && matchers.find((match) => match(filename)));
};
/** Inject typescript docgen information into modules at the end of a build */
class DocgenPlugin {
    constructor(options = {}) {
        this.name = "React Docgen Typescript Plugin";
        this.options = options;
    }
    apply(compiler) {
        const pluginName = "DocGenPlugin";
        const { docgenOptions, compilerOptions, generateOptions, } = this.getOptions();
        const docGenParser = docGen.withCompilerOptions(compilerOptions, docgenOptions);
        const { exclude = [], include = ["**/**.tsx"] } = this.options;
        const isExcluded = matchGlob(exclude);
        const isIncluded = matchGlob(include);
        compiler.hooks.compilation.tap(pluginName, (compilation, { normalModuleFactory }) => {
            compilation.dependencyTemplates.set(
            // eslint-disable-next-line
            // @ts-ignore TODO: Figure out why this isn't allowed
            dependency_1.default, new dependency_1.default.Template());
            const handler = (parser) => {
                parser.hooks.program.tap(pluginName, () => {
                    // eslint-disable-next-line
                    // @ts-ignore
                    const { module } = parser.state;
                    const nameForCondition = module.nameForCondition();
                    if (isExcluded(nameForCondition)) {
                        debugExclude(`Module not matched in "exclude": ${nameForCondition}`);
                        return;
                    }
                    if (!isIncluded(nameForCondition)) {
                        debugExclude(`Module not matched in "include": ${nameForCondition}`);
                        return;
                    }
                    const componentDocs = docGenParser.parse(nameForCondition);
                    module.addDependency(new dependency_1.default(module.request, generateDocgenCodeBlock_1.generateDocgenCodeBlock(Object.assign({ filename: nameForCondition, source: nameForCondition, componentDocs }, generateOptions)).substring(module.userRequest.length)));
                });
            };
            normalModuleFactory.hooks.parser
                .for("javascript/auto")
                .tap(pluginName, handler);
            normalModuleFactory.hooks.parser
                .for("javascript/dynamic")
                .tap(pluginName, handler);
            normalModuleFactory.hooks.parser
                .for("javascript/esm")
                .tap(pluginName, handler);
        });
    }
    getOptions() {
        const _a = this.options, { tsconfigPath = "./tsconfig.json", compilerOptions: userCompilerOptions, docgenCollectionName, setDisplayName, typePropName } = _a, docgenOptions = tslib_1.__rest(_a, ["tsconfigPath", "compilerOptions", "docgenCollectionName", "setDisplayName", "typePropName"]);
        let compilerOptions = {
            jsx: typescript_1.default.JsxEmit.React,
            module: typescript_1.default.ModuleKind.CommonJS,
            target: typescript_1.default.ScriptTarget.Latest,
        };
        if (userCompilerOptions) {
            compilerOptions = Object.assign(Object.assign({}, compilerOptions), userCompilerOptions);
        }
        else {
            const { options: tsOptions } = getTSConfigFile(tsconfigPath);
            compilerOptions = Object.assign(Object.assign({}, compilerOptions), tsOptions);
        }
        return {
            docgenOptions,
            generateOptions: {
                docgenCollectionName: docgenCollectionName || "STORYBOOK_REACT_CLASSES",
                setDisplayName: setDisplayName || true,
                typePropName: typePropName || "type",
            },
            compilerOptions,
        };
    }
}
exports.default = DocgenPlugin;
//# sourceMappingURL=plugin.js.map